/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.isobus.type;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.regex.Pattern;
import javax.xml.parsers.ParserConfigurationException;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObjectExistsException;
import org.openide.loaders.FileEntry;
import org.openide.loaders.MultiDataObject;
import org.openide.loaders.MultiDataObject.Entry;
import org.openide.loaders.MultiFileLoader;
import org.xml.sax.SAXException;

/**
 * File loader for Grid files Grid files are in the format
 * GRD[0-9][0-9][0-9][0-9][0-9].BIN or this format in lower case
 * 
 * Mandatory files : GRD[0-9][0-9][0-9][0-9][0-9].BIN or this format in lower case — grid format;
 * TASKDATA.XML or this format in lower case —the task data file which has references to the primary grid file;
 * 
 * Optional files : .TLG[0-9][0-9][0-9][0-9][0-9].BIN or this format in lower case — contains time log values; 
 *
 * @author Merijn Zengers
 */
public class GridFileLoader extends MultiFileLoader {

    public static final String PRIMARY_PATTERN = "[Gg][Rr][Dd][0-9][0-9][0-9][0-9][0-9]\\.[Bb][Ii][Nn]";
    public static final String TASKDATA_PATTERN = "[Tt][Aa][Ss][Kk][Dd][Aa][Tt][Aa]\\.[Xx][Mm][Ll]";
    public static final String TIMELOG_PATTERN = "[Tt][Ll][Gg][0-9][0-9][0-9][0-9][0-9]\\.[Bb][Ii][Nn]";
    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.isobus.type");

    public GridFileLoader() {
        super(GridDataObject.class.getName());
    }

    @Override
    protected String actionsContext() {
        StringBuilder builder = new StringBuilder("Loaders/");
        builder.append(GridDataObject.MIME_TYPE);
        builder.append("/Actions/");
        return builder.toString();
    }

    /**
     * Finds the primary file by one of the two secondary files The secondary
     * files are TASKDATA.XML or toLower And TLG[0-9][0-9][0-9][0-9][0-9].BIN or
     * toLower
     *
     * @param fo the fileObject to find the primary file by
     * @return if fo is one of the secondary files returns the primary file of
     * format GRD[0-9][0-9][0-9][0-9][0-9].BIN or toLower
     */
    @Override
    protected FileObject findPrimaryFile(FileObject fo) {
        // never recognize folders.
        if (fo.isFolder()) {
            return null;
        }
        Pattern primaryPattern = Pattern.compile(PRIMARY_PATTERN);
        Pattern taskDataPattern = Pattern.compile(TASKDATA_PATTERN);
        Pattern timeLogPattern = Pattern.compile(TIMELOG_PATTERN);
        if (primaryPattern.matcher(fo.getNameExt()).find()) {
            return fo;
        } else if (taskDataPattern.matcher(fo.getNameExt()).find() || timeLogPattern.matcher(fo.getNameExt()).find()) {
            for (FileObject child : fo.getParent().getChildren()) {
                if (!child.equals(fo)) {
                    Pattern childPattern = Pattern.compile(PRIMARY_PATTERN);
                    if (childPattern.matcher(child.getNameExt()).find()) {
                        return child;
                    }
                }
            }
        }
        LOGGER.log(Level.FINEST, "No primary file found for FileObject {0}", fo);
        return null;
    }

    /**
     * Creates the MultidataObject for the file with format
     * GRD[0-9][0-9][0-9][0-9][0-9].BIN or toLower() Will add TaskData and
     * TimeLog files as secondary files if present
     *
     * @param primaryFile the Primary file to create the DataObject for
     * @return the created DataObject
     * @throws DataObjectExistsException if the DataObject already exists
     * @throws IOException
     */
    @Override
    protected MultiDataObject createMultiObject(FileObject primaryFile) throws DataObjectExistsException, IOException {
        try {
            List<FileObject> secondary = new ArrayList<>();
            Pattern taskDataPattern = Pattern.compile(TASKDATA_PATTERN);
            Pattern timeLogPattern = Pattern.compile(TIMELOG_PATTERN);
            for (FileObject child : primaryFile.getParent().getChildren()) {
                if (!child.equals(primaryFile)) {
                    if (taskDataPattern.matcher(child.getNameExt()).find()) {
                        secondary.add(child);
                    } else if (timeLogPattern.matcher(child.getNameExt()).find()) {
                        secondary.add(child);
                    }
                }
            }
            GridDataObject dataObject = new GridDataObject(primaryFile, this, secondary);
            return dataObject;
        } catch (ParserConfigurationException | SAXException ex) {
            LOGGER.log(Level.FINEST, "An exception occured while tryint to create MultiDataObject {0} with exception {1}", new Object[]{primaryFile, ex});
        }
        return null;
    }

    /**
     * Creates a primary entry for the primary files
     *
     * @param obj the dataObject to create a primary entry for
     * @param primaryFile the primary file to add as primary entry
     * @return the created primary entry
     */
    @Override
    protected Entry createPrimaryEntry(MultiDataObject obj, FileObject primaryFile) {
        Entry entry = new FileEntry(obj, primaryFile);
        LOGGER.log(Level.FINEST, "Creating primary entry for {0}", primaryFile.getName());
        return entry;
    }

    /**
     * Creates a secondary entry for the provided MultiDataObject
     * @param obj the MultiDataObject to create the secondary entry for
     * @param secondaryFile the secondary file to create the entry from
     * @return The created entry
     */
    @Override
    protected Entry createSecondaryEntry(MultiDataObject obj, FileObject secondaryFile) {
        Pattern taskDataPattern = Pattern.compile(TASKDATA_PATTERN);
        Pattern timeLogPattern = Pattern.compile(TIMELOG_PATTERN);
        assert (taskDataPattern.matcher(secondaryFile.getNameExt()).find() || timeLogPattern.matcher(secondaryFile.getNameExt()).find());

        FileEntry entry = new FileEntry(obj, secondaryFile);
        LOGGER.log(Level.FINEST, "Creating secondary entry for {0}", secondaryFile.getName());
        return entry;
    }
}
